package com.qmth.themis.backend.api;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.themis.backend.util.ServletUtil;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.AuthDto;
import com.qmth.themis.business.entity.TBSession;
import com.qmth.themis.business.entity.TBUser;
import com.qmth.themis.business.enums.RoleEnum;
import com.qmth.themis.business.service.EhcacheService;
import com.qmth.themis.business.service.TBSessionService;
import com.qmth.themis.business.service.TBUserService;
import com.qmth.themis.business.util.JwtUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.business.util.SessionUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.enums.Platform;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.AesUtil;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 用户 前端控制器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Api(tags = "用户Controller")
@RestController
@RequestMapping("/${prefix.url.admin}/user")
public class TBUserController {
    private final static Logger log = LoggerFactory.getLogger(TBUserController.class);

    @Resource
    TBUserService tbUserService;

    @Resource
    TBSessionService tbSessionService;

    @Resource
    EhcacheService ehcacheService;

    @ApiOperation(value = "用户登录接口")
    @RequestMapping(value = "/login/account", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "用户信息", response = TBUser.class)})
    public Result login(@ApiParam(value = "用户信息", required = true) @RequestBody TBUser tbUser, HttpServletRequest request) throws NoSuchAlgorithmException, UnsupportedEncodingException, InvalidKeySpecException {
        if (Objects.isNull(tbUser.getLoginName()) || Objects.equals(tbUser.getLoginName(), "")) {
            throw new BusinessException(ExceptionResultEnum.LOGIN_NAME_IS_NULL);
        }
        if (Objects.isNull(tbUser.getPassword()) || Objects.equals(tbUser.getPassword(), "")) {
            throw new BusinessException(ExceptionResultEnum.PASSWORD_IS_NULL);
        }
        String loginName = tbUser.getLoginName();
        String password = tbUser.getPassword();

        QueryWrapper<TBUser> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(TBUser::getLoginName, loginName);
        TBUser user = tbUserService.getOne(wrapper);
        //用户不存在
        if (Objects.isNull(user)) {
            throw new BusinessException(ExceptionResultEnum.USER_NO);
        }
        String loginPassword = AesUtil.decryptCs7(password, Constants.AES_RULE);
        //密码错误
        String aesPassword = AesUtil.decryptCs7(user.getPassword(), Constants.AES_RULE);
        if (!Objects.equals(loginPassword, aesPassword)) {
            throw new BusinessException(ExceptionResultEnum.PASSWORD_NO);
        }
        //停用
        if (user.getEnable().intValue() == 0) {
            throw new BusinessException(ExceptionResultEnum.USER_ENABLE);
        }
        Platform platform = Platform.valueOf(ServletUtil.getRequestPlatform(request));
        String deviceId = ServletUtil.getRequestDeviceId(request);
        //添加用户鉴权缓存
        AuthDto authDto = ehcacheService.addAccountCache(user);
        //生成token
        String token = JwtUtil.sign(user.getId(), platform, deviceId, authDto.getRoleEnum());
        //添加用户缓存
        RedisUtil.setUser(user.getId(), user);
        //添加用户会话缓存
        String sessionId = SessionUtil.digest(user.getId(), authDto.getRoleEnum().name(), platform.getSource());
        TBSession tbSession = tbSessionService.saveSessionInfo(sessionId, user.getId(), authDto.getRoleEnum().name(), platform.name(), platform.getSource(), deviceId, token, request.getLocalAddr());
        RedisUtil.setUserSession(sessionId, tbSession);

        Map<String, Object> map = new HashMap<>();
        map.put(SystemConstant.TOKEN, token);
        map.put(SystemConstant.USER, user);
        return ResultUtil.ok(map);
    }

    @ApiOperation(value = "用户查询接口")
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public Result list() {
        return ResultUtil.ok(SystemConstant.SUCCESS);
    }

    @ApiOperation(value = "登出接口")
    @RequestMapping(value = "/logout", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    public Result logout(HttpServletRequest request) throws NoSuchAlgorithmException {
        String token = ServletUtil.getRequestToken(request);
        if (Objects.isNull(token) || Objects.equals(token, "")) {
            throw new BusinessException(ExceptionResultEnum.TOKEN_INVALID);
        }
        Platform platform = Platform.valueOf(ServletUtil.getRequestPlatform(request));
        if (Objects.isNull(platform) || Objects.equals(platform, "")) {
            throw new BusinessException(ExceptionResultEnum.PLATFORM_INVALID);
        }
        String userId = JwtUtil.getClaim(token, SystemConstant.JWT_USERID);
        String role = JwtUtil.getClaim(token, SystemConstant.ROLE);
        String sessionId = SessionUtil.digest(Long.parseLong(userId), RoleEnum.valueOf(role), platform.getSource());
        RedisUtil.deleteUserSession(sessionId);
        return ResultUtil.ok(JSONObject.parseObject(SystemConstant.SUCCESS));
    }
}
