package com.qmth.themis.backend.interceptor;

import com.qmth.themis.backend.config.DictionaryConfig;
import com.qmth.themis.backend.util.ServletUtil;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.AuthDto;
import com.qmth.themis.business.entity.TBSession;
import com.qmth.themis.business.entity.TBUser;
import com.qmth.themis.business.enums.RoleEnum;
import com.qmth.themis.business.service.EhcacheService;
import com.qmth.themis.business.service.TBUserService;
import com.qmth.themis.business.util.EhcacheUtil;
import com.qmth.themis.business.util.JwtUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.business.util.SessionUtil;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.enums.Platform;
import com.qmth.themis.common.exception.BusinessException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Set;

/**
 * @Description: 鉴权拦截器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/27
 */
public class AuthInterceptor implements HandlerInterceptor {
    private final static Logger log = LoggerFactory.getLogger(AuthInterceptor.class);

    @Resource
    EhcacheService ehcacheService;

    @Resource
    DictionaryConfig dictionaryConfig;

    @Resource
    TBUserService tbUserService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object object) throws Exception {
        log.info("HandlerInterceptor preHandle is come in");
        String url = request.getServletPath();
        if (url.equalsIgnoreCase("/error")) {
            if (response.getStatus() == cn.hutool.http.HttpStatus.HTTP_NOT_FOUND) {
                throw new BusinessException(ExceptionResultEnum.NOT_FOUND);
            } else {
                throw new BusinessException(ExceptionResultEnum.EXCEPTION_ERROR);
            }
        }
        String token = ServletUtil.getRequestToken(request);
        Platform platform = Platform.valueOf(ServletUtil.getRequestPlatform(request));
        String deviceId = ServletUtil.getRequestDeviceId(request);
        if (Objects.isNull(token) || Objects.equals(token, "")) {
            throw new BusinessException(ExceptionResultEnum.TOKEN_INVALID);
        }
        if (Objects.isNull(platform) || Objects.equals(platform, "")) {
            throw new BusinessException(ExceptionResultEnum.PLATFORM_INVALID);
        }
        if (Objects.isNull(deviceId) || Objects.equals(deviceId, "")) {
            throw new BusinessException(ExceptionResultEnum.DEVICE_ID_INVALID);
        }
        Long userId = Long.parseLong(JwtUtil.getClaim(token, SystemConstant.JWT_USERID));
        String role = JwtUtil.getClaim(token, SystemConstant.ROLE);
        //首先验证token是否匹配
        if (!JwtUtil.verify(token, userId, platform, deviceId, RoleEnum.valueOf(role))) {
            throw new BusinessException(ExceptionResultEnum.TOKEN_NO);
        }
        //系统公用接口不拦截
        List<String> sysUrls = dictionaryConfig.systemUrlDomain().getUrls();
        int sysCount = (int) sysUrls.stream().filter(s -> {
            return s.equalsIgnoreCase(url);
        }).count();
        if (sysCount > 0) {
            return true;
        }
        //验证token是否有效
        String sessionId = SessionUtil.digest(userId, RoleEnum.valueOf(role), platform.getSource());
        TBSession tbSession = (TBSession) RedisUtil.getUserSession(sessionId);
        if (Objects.isNull(tbSession)) {
            throw new BusinessException(ExceptionResultEnum.LOGIN_NO);
        } else {
            if (!Objects.equals(token, tbSession.getAccessToken())) {
                throw new BusinessException(ExceptionResultEnum.TOKEN_NO);
            }
            Date expireTime = tbSession.getExpireTime();
            //手机端的token时长为一个月，所以会出现缓存没有的情况
            TBUser tbUser = (TBUser) RedisUtil.getUser(userId);
            if (Objects.isNull(tbUser)) {
                tbUser = tbUserService.getById(userId);
                RedisUtil.setUser(tbUser.getId(), tbUser);
            }
            //还剩5分钟刷新会话缓存
            if (Objects.nonNull(expireTime) && (expireTime.getTime() - System.currentTimeMillis()) <= SystemConstant.REFRESH_EXPIRE_TIME) {
                RedisUtil.refreshUserSession(sessionId);
            }
        }
        AuthDto authDto = (AuthDto) EhcacheUtil.get(SystemConstant.AUTH_CACHE, userId);
        if (Objects.isNull(authDto)) {
            authDto = ehcacheService.addAccountCache(userId);
        }
        //验证权限
        Set<String> urls = authDto.getUrls();
        int count = (int) urls.stream().filter(s -> {
            return s.equalsIgnoreCase(url);
        }).count();
        if (count == 0) {
            throw new BusinessException(ExceptionResultEnum.UN_AUTHORIZATION);
        }
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request,
                           HttpServletResponse response,
                           Object o, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest request,
                                HttpServletResponse response,
                                Object o, Exception e) throws Exception {
    }
}
